@@ -120,7 +120,7 @@ static const char edit_usage_str_2[] =
 "                        -N options must be last global options.\n";
 
 static const char edit_usage_str_3[] =
-"  --net               - allow network access"
+"  --net               - allow network access\n"
 "  --help or -h        - display help\n\n"
 "where <action>\n"
 "  -d or --delete <xpath>\n"
@@ -216,7 +216,7 @@ void
 nsarr_xpath_register(xmlXPathContextPtr ctxt)
 {
     int ns;
-    for (ns = 0; ns_arr[ns]; ns++) {
+    for (ns = 0; ns_arr[ns]; ns += 2) {
         xmlXPathRegisterNs(ctxt, ns_arr[ns], ns_arr[ns+1]);
     }
 }
@@ -355,6 +355,10 @@ edMove(xmlDocPtr doc, xmlNodeSetPtr nodes, xmlNodePtr to)
     int i;
     for (i = 0; i < nodes->nodeNr; i++)
     {
+        if (nodes->nodeTab[i]->type == XML_NAMESPACE_DECL) {
+            fprintf(stderr, "FIXME: can't move namespace nodes\n");
+            exit(EXIT_INTERNAL_ERROR);
+        }
         /* move node */
         xmlUnlinkNode(nodes->nodeTab[i]);
         xmlAddChild(to, nodes->nodeTab[i]);
@@ -379,7 +383,7 @@ edProcess(xmlDocPtr doc, XmlEdAction* ops, int ops_count)
         xmlNodeSetPtr nodes;
 
         res = xmlXPathEvalExpression(BAD_CAST ops[k].arg1, ctxt);
-        if (!res || res->type != XPATH_NODESET) continue;
+        if (!res || res->type != XPATH_NODESET || !res->nodesetval) continue;
         nodes = res->nodesetval;
 
         switch (ops[k].op)
@@ -431,7 +435,9 @@ edOutput(const char* filename, edOptions g_ops)
 {
     xmlDocPtr doc;
     int save_options =
+#if LIBXML_VERSION >= 20708
         (g_ops.noblanks? 0 : XML_SAVE_WSNONSIG) |
+#endif
         (g_ops.preserveFormat? 0 : XML_SAVE_FORMAT) |
         (g_ops.omit_decl? XML_SAVE_NO_DECL : 0);
     int read_options =
