@@ -90,7 +90,8 @@ typedef struct _selOptions {
 
 typedef selOptions *selOptionsPtr;
 
-typedef enum { TARG_NONE = 0, TARG_SORT_OP, TARG_XPATH, TARG_STRING,
+typedef enum { TARG_NONE = 0, TARG_SORT_OP, TARG_XPATH,
+               TARG_ATTR_STRING, TARG_STRING,
                TARG_NEWLINE, TARG_NO_CMDLINE = TARG_NEWLINE, TARG_INP_NAME
 } template_argument_type;
 typedef struct {
@@ -177,8 +178,8 @@ static const template_option
     OPT_IF       = { 'i', "if", BAD_CAST"when", {{BAD_CAST "test", TARG_XPATH}}, 1 },
     OPT_ELIF     = { 0,   "elif", BAD_CAST "when", {{BAD_CAST "test", TARG_XPATH}}, 1 },
     OPT_ELSE     = { 0,   "else", BAD_CAST "otherwise", {{NULL}}, 1 },
-    OPT_ELEM     = { 'e', "elem", BAD_CAST "element", {{BAD_CAST "name", TARG_XPATH}}, 1 },
-    OPT_ATTR     = { 'a', "attr", BAD_CAST "attribute", {{BAD_CAST "name", TARG_XPATH}}, 1 },
+    OPT_ELEM     = { 'e', "elem", BAD_CAST "element", {{BAD_CAST "name", TARG_ATTR_STRING}}, 1 },
+    OPT_ATTR     = { 'a', "attr", BAD_CAST "attribute", {{BAD_CAST "name", TARG_ATTR_STRING}}, 1 },
     OPT_BREAK    = { 'b', "break", NULL, {{NULL}}, -1 },
     OPT_SORT     = { 's', "sort", BAD_CAST "sort", {{NULL, TARG_SORT_OP}, {BAD_CAST "select", TARG_XPATH}}, 0 },
 
@@ -416,7 +417,7 @@ selGenTemplate(xmlNodePtr root, xmlNodePtr template_node,
         const template_option *newtarg = NULL;
         int j;
 
-        if (argv[i][0] == '-')
+        if (argv[i][0] == '-' && argv[i][1] != '\0')
         {
             for (j = 0; j < sizeof(TEMPLATE_OPTIONS)/sizeof(*TEMPLATE_OPTIONS); j++)
             {
@@ -472,11 +473,11 @@ selGenTemplate(xmlNodePtr root, xmlNodePtr template_node,
                 selUsage(argv[0], EXIT_BAD_ARGS);
             switch (newtarg->arguments[j].type)
             {
-            case TARG_XPATH: {
+            case TARG_XPATH:
                 checkNsRefs(root, argv[i]);
+            case TARG_ATTR_STRING:
                 xmlNewProp(newnode, newtarg->arguments[j].attrname, BAD_CAST argv[i]);
                 break;
-            }
 
             case TARG_STRING:
                 xmlNodeAddContent(newnode, BAD_CAST argv[i]);
@@ -669,14 +670,13 @@ selPrepareXslt(xmlDocPtr style, selOptionsPtr ops, xmlChar *ns_arr[],
 }
 
 /**
- * copy top-level namespace definitions from @doc to @style_tree
+ * copy namespace definitions from @root to @style_tree
  */
 static void
-extract_ns_defs(xmlDocPtr doc, xmlDocPtr style_tree)
+extract_ns_defs(xmlNodePtr root, xmlDocPtr style_tree)
 {
     xmlNsPtr nsDef;
     xmlNodePtr style_root = xmlDocGetRootElement(style_tree);
-    xmlNodePtr root = xmlDocGetRootElement(doc);
     if (!root) return;
 
     for (nsDef = root->nsDef; nsDef; nsDef = nsDef->next) {
@@ -706,7 +706,7 @@ do_file(const char *filename, xmlDocPtr style_tree,
 
         static xsltStylesheetPtr style = NULL;
         if (!style) {
-            extract_ns_defs(doc, style_tree);
+            extract_ns_defs(xmlDocGetRootElement(doc), style_tree);
             /* Parse XSLT stylesheet */
             style = xsltParseStylesheetDoc(style_tree);
             if (!style) exit(EXIT_LIB_ERROR);
@@ -761,6 +761,12 @@ selMain(int argc, char **argv)
 
     if (ops.printXSLT)
     {
+        if (i < argc) {
+            xmlTextReaderPtr reader = xmlReaderForFile(argv[i], NULL, xml_options);
+            xmlTextReaderRead(reader);
+            extract_ns_defs(xmlTextReaderCurrentNode(reader), style_tree);
+            xmlTextReaderClose(reader);
+        }
         xmlDocFormatDump(stdout, style_tree, 1);
         exit(EXIT_SUCCESS);
     }
